@@ -28,7 +28,8 @@ void new_json_obj(int json)
 			perror("json object");
 			exit(1);
 		}
-		jsonw_pretty(_jw, true);
+		if (pretty)
+			jsonw_pretty(_jw, true);
 		jsonw_start_array(_jw);
 	}
 }
@@ -88,9 +89,7 @@ void open_json_array(enum output_type type, const char *str)
 void close_json_array(enum output_type type, const char *str)
 {
 	if (_IS_JSON_CONTEXT(type)) {
-		jsonw_pretty(_jw, false);
 		jsonw_end_array(_jw);
-		jsonw_pretty(_jw, true);
 	} else if (_IS_FP_CONTEXT(type)) {
 		printf("%s", str);
 	}
@@ -117,8 +116,11 @@ void close_json_array(enum output_type type, const char *str)
 		}							\
 	}
 _PRINT_FUNC(int, int);
+_PRINT_FUNC(s64, int64_t);
 _PRINT_FUNC(hu, unsigned short);
-_PRINT_FUNC(uint, uint64_t);
+_PRINT_FUNC(uint, unsigned int);
+_PRINT_FUNC(u64, uint64_t);
+_PRINT_FUNC(luint, unsigned long int);
 _PRINT_FUNC(lluint, unsigned long long int);
 _PRINT_FUNC(float, double);
 #undef _PRINT_FUNC
